/*-------------------------------------------------------------------------+
| Copyright 2015 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.annotation.editingsupport;

import java.util.Collection;
import java.util.List;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.EEnum;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.jface.viewers.ColumnViewer;
import org.eclipse.jface.viewers.EditingSupport;
import org.fortiss.tooling.base.annotation.valueprovider.DerivedAnnotationValueProviderBase;
import org.fortiss.tooling.base.annotation.valueprovider.EStructuralFeatureDescriptor;
import org.fortiss.tooling.base.annotation.valueprovider.EStructuralFeatureDescriptor.EReferenceScope;
import org.fortiss.tooling.base.annotation.valueprovider.EStructuralFeatureValueProviderBase;
import org.fortiss.tooling.base.annotation.valueprovider.IAnnotationValueProvider;
import org.fortiss.tooling.base.annotation.valueprovider.ValueProviderBase;
import org.fortiss.tooling.base.model.element.IAnnotatedSpecification;
import org.fortiss.tooling.base.model.element.IDerivedAnnotation;
import org.fortiss.tooling.base.ui.annotation.view.generic.GenericAnnotationView;

/**
 * Factory to build {@link AnnotationEditingSupportBase}-based {@link EditingSupport} for the
 * {@link GenericAnnotationView}.
 * 
 * @author barner
 */
public class EditingSupportFactory {

	/**
	 * Creates the {@link EditingSupport} that is directly based on
	 * {@link DerivedAnnotationValueProviderBase}.
	 */
	private static <T extends IDerivedAnnotation<?>, V extends DerivedAnnotationValueProviderBase<T>>
			EditingSupport createEditingSupport4DerivedAnnotationValueProviderBase(V valueProvider,
					ColumnViewer viewer, Class<? extends IAnnotatedSpecification> clazz,
					T specification) throws Exception {

		if(!valueProvider.isEditableByUser()) {
			return null;
		}

		return createEditingSupport4EStructuralFeatureValueProviderBase(
				(EStructuralFeatureValueProviderBase<T>)valueProvider, viewer, clazz,
				specification);
	}

	/**
	 * Creates the {@link EditingSupport} that is directly based on
	 * {@link EStructuralFeatureValueProviderBase}.
	 * 
	 * @param valueProvider
	 *            Annotation's {@link IAnnotationValueProvider}
	 * @param viewer
	 *            {@link ColumnViewer} used to display annotation
	 * @param clazz
	 *            Annotation type
	 * @param specification
	 *            Annotation instance
	 * @return {@link EditingSupport} that is directly based on
	 *         {@link EStructuralFeatureValueProviderBase}.
	 */
	private static <T extends IAnnotatedSpecification, V extends EStructuralFeatureValueProviderBase<T>>
			EditingSupport createEditingSupport4EStructuralFeatureValueProviderBase(V valueProvider,
					ColumnViewer viewer, Class<? extends IAnnotatedSpecification> clazz,
					T specification) throws Exception {

		EStructuralFeature structuralFeature = valueProvider.getEStructuralFeatureDescriptor()
				.getEStructuralFeature(specification);

		if(structuralFeature == null) {
			throw new Exception("EStructuralValueProvider: feature has not been set.");
		}
		if(structuralFeature.getUpperBound() == 0) {
			throw new Exception(
					"EStructuralValueProvider: feature multiplicity == 0 is not supported.");
		}

		EClassifier eType = structuralFeature.getEType();
		EStructuralFeatureDescriptor eStructuralFeatureDescriptor =
				valueProvider.getEStructuralFeatureDescriptor();
		if(structuralFeature.getUpperBound() == 1) {

			EReferenceScope eReferenceScope = eStructuralFeatureDescriptor.getEReferenceScope();
			boolean isSimpleType = (!(eType instanceof EEnum)) && (!(eType instanceof EClass));
			if(isSimpleType || ((eType instanceof EClass) && (eReferenceScope == null))) {

				return createEditingSupport4ValueProviderBase((ValueProviderBase<T>)valueProvider,
						viewer, clazz, specification);
			}
			return new ComboBoxEditingSupport(viewer, clazz, specification,
					eStructuralFeatureDescriptor);
		}

		if(eType instanceof EClass || eType instanceof EEnum) {
			return new MultiSelectionEditingSupport(viewer, clazz, specification,
					eStructuralFeatureDescriptor);
		}

		throw new Exception(
				"For feature multiplicity > 1, EStructuralValueProvider has been implemented for EReferences and EEnumns, only .");
	}

	/**
	 * Creates the {@link EditingSupport} for value providers that are directly based on
	 * {@link ValueProviderBase}.
	 * 
	 * @param valueProvider
	 *            Annotation's {@link IAnnotationValueProvider}
	 * @param viewer
	 *            {@link ColumnViewer} used to display annotation
	 * @param clazz
	 *            Annotation type
	 * @param specification
	 *            Annotation instance
	 * @return {@link EditingSupport} for value providers that are directly based on
	 *         {@link ValueProviderBase}.
	 */
	@SuppressWarnings("unchecked")
	private static <T extends IAnnotatedSpecification, V extends ValueProviderBase<T>>
			EditingSupport createEditingSupport4ValueProviderBase(V valueProvider,
					ColumnViewer viewer, Class<? extends IAnnotatedSpecification> clazz,
					T specification) throws Exception {

		try {
			valueProvider.updateInputChoice(viewer.getInput(), (Class<T>)clazz);
		} catch(Exception e) {
			// Use standard text editor as fall-back
			valueProvider.setCurrentInputChoice(null);
		}

		// Input is not restricted -> free editing in text cell
		Collection<String> currentInputChoice = valueProvider.getCurrentInputChoice();
		if(currentInputChoice == null) {

			if(valueProvider.getAnnotationValue(specification) instanceof Boolean) {
				return new CheckBoxEditingSupport(viewer, clazz);
			}

			return new TextEditingSupport(viewer, clazz);
		}

		// Input is restricted to concrete set of values
		// (use SingleEnumAttributeValueProviderBase for EEnum types!)
		List<String> fixedInputChoice = valueProvider.getFixedInputChoice();
		return new ComboBoxEditingSupport(viewer, clazz, fixedInputChoice.isEmpty());
	}

	/**
	 * Creates an {@link EditingSupport} for the given {@link IAnnotatedSpecification}.
	 * 
	 * @param valueProvider
	 *            Annotation's {@link IAnnotationValueProvider}
	 * @param viewer
	 *            {@link ColumnViewer} used to display annotation
	 * @param clazz
	 *            Annotation type
	 * @param specification
	 *            Annotation instance
	 * @return {@link EditingSupport} for the given {@link IAnnotatedSpecification}.
	 */
	@SuppressWarnings({"rawtypes", "unchecked"})
	public static <T extends IAnnotatedSpecification> EditingSupport createEditingSupport(
			IAnnotationValueProvider<T> valueProvider, ColumnViewer viewer,
			Class<? extends IAnnotatedSpecification> clazz, T specification) throws Exception {

		if(valueProvider instanceof DerivedAnnotationValueProviderBase<?>) {
			return createEditingSupport4DerivedAnnotationValueProviderBase(
					(DerivedAnnotationValueProviderBase)valueProvider, viewer, clazz,
					(IDerivedAnnotation)specification);
		} else if(valueProvider instanceof EStructuralFeatureValueProviderBase<?>) {
			return createEditingSupport4EStructuralFeatureValueProviderBase(
					(EStructuralFeatureValueProviderBase<T>)valueProvider, viewer, clazz,
					specification);
		}
		if(valueProvider instanceof ValueProviderBase<?>) {
			return createEditingSupport4ValueProviderBase((ValueProviderBase<T>)valueProvider,
					viewer, clazz, specification);
		}

		return null;
	}
}
