/*-------------------------------------------------------------------------+
| Copyright 2011 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.utils;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.model.IIdLabeled;
import org.fortiss.tooling.kernel.model.IIdLabeledReference;
import org.fortiss.tooling.kernel.model.ILibraryElementReference;
import org.fortiss.tooling.kernel.model.INamedCommentedElement;
import org.fortiss.tooling.kernel.model.INamedElement;
import org.fortiss.tooling.kernel.model.IProjectRootElement;
import org.fortiss.tooling.kernel.service.ICommandStackService;
import org.fortiss.tooling.kernel.service.IPersistencyService;

/**
 * Utility methods for accessing {@link IProjectRootElement}s.
 * 
 * @author hoelzl
 */
public final class KernelModelElementUtils {

	/**
	 * Returns the instance of a project root element corresponding to the given
	 * class.
	 * 
	 * @param element
	 *            a model element contained in the project to be searched for
	 *            the root element.
	 * @param clazz
	 *            the class of the root element to be returned.
	 * @return instance of class <code>clazz</code> or <code>null</code> if no
	 *         such root element was found.
	 */
	public static <T extends IProjectRootElement> T getRootElement(EObject element,
			Class<T> clazz) {
		EObject root = getRootElementContainer(element);
		return EcoreUtils.pickFirstInstanceOf(clazz, root.eContents());
	}

	/**
	 * Returns the list of instances of a project root element corresponding to the given
	 * class.
	 * 
	 * @param element
	 *            a model element contained in the project to be searched for
	 *            the root element.
	 * @param clazz
	 *            the class of the root element to be returned.
	 * @return list of instances of class <code>clazz</code> or <code>null</code> if no
	 *         such root element was found.
	 */
	public static <T extends IProjectRootElement> List<T> getRootElements(EObject element,
			Class<T> clazz) {
		EObject root = getRootElementContainer(element);
		return EcoreUtils.pickInstanceOf(clazz, root.eContents());
	}

	/**
	 * Returns the root of an element.
	 * 
	 * @param element
	 *            a model element contained in the project to be searched for
	 *            the root element.
	 * @return root of the given element.
	 */
	private static <T extends IProjectRootElement> EObject
			getRootElementContainer(EObject element) {
		ITopLevelElement topelement =
				IPersistencyService.getInstance().getTopLevelElementFor(element);
		EObject root;
		if(topelement == null) {
			root = element;
			while(root.eContainer() != null)
				root = root.eContainer();
		} else {
			root = topelement.getRootModelElement();
		}

		return root;
	}

	/**
	 * For a given element, returns the closest ancestor with a certain type.
	 * 
	 * @param element
	 *            a model element.
	 * @param targetClass
	 *            the class of the parent that is returned.
	 * @param isReflexive
	 *            states how to handle the case where <code>element</code> itself is of type
	 *            <code>targetClass</code>: if <code>isReflexive</code> is true then the element is
	 *            returned, otherwise the closest parent of type <code>targetClass</code> is
	 *            returned
	 * 
	 * @return instance of class <code>targetClass</code> or <code>null</code> if no such parent
	 *         element was found.
	 */
	@SuppressWarnings("unchecked")
	public static <T extends EObject> T getParentElement(EObject element, Class<T> targetClass,
			boolean isReflexive) {
		if(element == null) {
			return null;
		}
		EObject currentParent = isReflexive ? element : element.eContainer();
		while(currentParent != null && !(targetClass.isAssignableFrom(currentParent.getClass()))) {
			currentParent = currentParent.eContainer();
		}
		return (T)currentParent;
	}

	/**
	 * Returns the {@link List} of all parent elements of the given {@link EObject} that
	 * inherit from the given target {@link Class} (possibly including the element itself).
	 * 
	 * @param element
	 *            the element whose parents should be determined (possibly including the element
	 *            itself)
	 * 
	 * @param targetClass
	 *            the {@link Class} "filter"
	 * 
	 * @return {@link List} of all parent elements of the given {@link EObject} that
	 *         inherit from the given target {@link Class} (possibly including the element itself)
	 */
	@SuppressWarnings("unchecked")
	public static <T extends EObject> List<T> getParentElements(EObject element,
			Class<T> targetClass) {
		List<T> retList = new ArrayList<T>();
		EObject currentElement = element;
		while(currentElement != null && !(currentElement instanceof IProjectRootElement)) {
			if(targetClass.isAssignableFrom(currentElement.getClass())) {
				retList.add((T)currentElement);
			}
			currentElement = currentElement.eContainer();
		}
		return retList;
	}

	/**
	 * Finds the model element referenced by the given {@link IIdLabeledReference} element or
	 * <code>null</code> if no such element exists. The reference's model project is searched from
	 * its root.
	 * 
	 * @param reference
	 *            the ID-reference to be searched
	 * @return the referenced element or <code>null</code>.
	 */
	public static EObject findReferencedElement(IIdLabeledReference reference) {
		ITopLevelElement topelement =
				IPersistencyService.getInstance().getTopLevelElementFor(reference);
		return findElementById(reference.getIdReference(), topelement.getRootModelElement());
	}

	/**
	 * Finds the model element referenced by the given {@link IIdLabeledReference} element or
	 * <code>null</code> if no such element exists.
	 * 
	 * @param id
	 *            the ID of the element to be searched.
	 * @param root
	 *            the root model element to be searched.
	 * @return the referenced element or <code>null</code>.
	 */
	public static IIdLabeled findElementById(int id, EObject root) {
		if(root instanceof IIdLabeled && ((IIdLabeled)root).getId() == id) {
			return (IIdLabeled)root;
		}
		for(Iterator<EObject> iter = root.eAllContents(); iter.hasNext();) {
			EObject eo = iter.next();
			if(eo instanceof IIdLabeled && ((IIdLabeled)eo).getId() == id) {
				return (IIdLabeled)eo;
			}
		}
		return null;
	}

	/**
	 * Determines if the given element is an indirect child of the given parent
	 * element.
	 * 
	 * @param element
	 *            the child model element.
	 * @param parentElement
	 *            the parent model element.
	 * @return <code>true</code> if the element is a child in the aggregation
	 *         tree of the parent element.
	 */
	public static boolean isChildElementOf(EObject element, EObject parentElement) {
		while(element.eContainer() != null) {
			if(element.eContainer() == parentElement) {
				return true;
			}
			element = element.eContainer();
		}
		return false;
	}

	/**
	 * Keeps only the top-most parents of a list. I.e., if the list contains two elements, one is
	 * the (direct or indirect) child of the other, then it is removed.
	 */
	public static EList<EObject> topParents(EList<EObject> objs) {
		EList<EObject> res = new BasicEList<EObject>();
		for(EObject o1 : objs) {
			boolean hasParent = false;
			for(EObject o2 : objs) {
				if(isChildElementOf(o1, o2)) {
					hasParent = true;
					break;
				}
			}
			if(!hasParent) {
				res.add(o1);
			}
		}
		return res;
	}

	/**
	 * Iterates the content of the given root element and finds a contained element with the given
	 * name and the given class.
	 * 
	 * @param root
	 *            the root element to be searched.
	 * @param name
	 *            the name of the element to find.
	 * @param clazz
	 *            the class of the element to find.
	 * @return the element found or <code>null</code> if none was found.
	 */
	@SuppressWarnings("unchecked")
	public static <T extends INamedElement> T findContentElementByNameAndClass(EObject root,
			String name, Class<T> clazz) {
		Iterator<EObject> iter = root.eAllContents();
		while(iter.hasNext()) {
			EObject nxt = iter.next();
			if(clazz.isInstance(nxt) && ((T)nxt).getName().equals(name)) {
				return (T)nxt;
			}
		}
		return null;
	}

	/**
	 * Internal helper method for {@link #computeFullyQualifiedName(INamedElement, boolean)} and
	 * {@link #computeRelativeName(INamedElement, INamedElement)} that
	 * prepends the name of a single element to the current value of fully qualified name.
	 */
	private static void prependName(StringBuilder sb, EObject current) {
		if(current instanceof INamedElement && !(current instanceof ILibraryElementReference)) {

			if(sb.length() != 0) {
				sb.insert(0, ".");
			}
			sb.insert(0, ((INamedElement)current).getName());
		}
	}

	/**
	 * Computes the qualified name relative to a parent element.
	 * 
	 * @param parent
	 *            the parent element.
	 * @param element
	 *            the element whose relative name will be computed.
	 * @return the qualified name relative to the parent element or null if the
	 *         parent element is not a container for element.
	 */
	public static String computeRelativeName(INamedElement parent, INamedElement element) {
		StringBuilder sb = new StringBuilder();
		EObject current = element;

		while(current != null && current != parent) {
			prependName(sb, current);
			current = current.eContainer();
		}
		if(current == null) {
			return null;
		}

		return sb.toString();
	}

	/**
	 * Computes the fully qualified name of an element.
	 * 
	 * @param element
	 *            the element whose fully qualified name will be computed.
	 * 
	 * @return the element's fully qualified name.
	 */
	public static String computeFullyQualifiedName(INamedElement element) {
		return computeFullyQualifiedName(element, true);
	}

	/**
	 * Computes the fully qualified name of an element.
	 * 
	 * @param element
	 *            the element whose fully qualified name will be computed.
	 * 
	 * @param includeTopLevel
	 *            Flag whether to include the top-level of the model element hierarchy into the name
	 * 
	 * @return the element's fully qualified name.
	 */
	public static String computeFullyQualifiedName(INamedElement element, boolean includeTopLevel) {
		StringBuilder sb = new StringBuilder();
		EObject current = element;

		while(current.eContainer() != null) {
			prependName(sb, current);
			current = current.eContainer();
		}
		if(includeTopLevel) {
			prependName(sb, current);
		}

		return sb.toString();
	}

	/**
	 * Returns true if the parameter is a child of a {@link ILibraryElementReference}.
	 * 
	 * @param eObject
	 *            an object from the model.
	 * 
	 * @return <code>true</code> if the element is under a library reference.
	 */
	public static boolean isChildOfLibraryReference(EObject eObject) {
		if(eObject != null) {
			int count = 0;
			for(EObject parent = eObject.eContainer(); parent != null; parent =
					parent.eContainer()) {
				if(parent instanceof ILibraryElementReference) {
					return true;
				}
				if(++count > 100000) {
					return isChildOfLibraryReference(parent);
				}
				if(parent == eObject) {
					throw new IllegalStateException(
							"There is a cycle in the containment hierarchy of " + eObject);
				}
			}
		}
		return false;
	}

	/**
	 * Executes model-changing {@link Runnable} on the given element's command stack. This method
	 * should only be used for {@link Runnable}s, which actually modify the model.
	 * 
	 * @param element
	 *            a model element, which is used to find the persisted model
	 *            project.
	 * @param command
	 *            the command to be executed.
	 */
	public static void runAsCommand(EObject element, Runnable command) {
		ICommandStackService.getInstance().runAsCommand(element, command);
	}

	/**
	 * @param t
	 *            the model element to set name and comment
	 * @param name
	 *            the name
	 * @param comment
	 *            the comment
	 * @return <code>t</code> whose name and comment are set to <code>name</code> and
	 *         <code>comment</code>
	 */
	public static <T extends INamedCommentedElement> T setNameComment(T t, String name,
			String comment) {
		t.setName(name);
		t.setComment(comment);
		return t;
	}
}
