/*-------------------------------------------------------------------------+
|                                                                          |
| Copyright 2005-2011 The ConQAT Project                                   |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+-------------------------------------------------------------------------*/
package org.conqat.ide.commons.ui.logging;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Plugin;
import org.eclipse.core.runtime.Status;

import org.conqat.ide.commons.ui.dialog.MessageUtils;

/**
 * Utility code for performing logging.
 * 
 * @author hummelb
 * @author schwitze
 * @author $Author: deissenb $
 * @version $Rev: 34252 $
 * @levd.rating GREEN Hash: AA5D7661DFC5CC4BF4D59C1A84E5D471
 */
public class LoggingUtils {

	/**
	 * Logs a message that occurred in the context of the given plug-in with the
	 * given severity.
	 * 
	 * @see IStatus
	 * 
	 * @param plugin
	 *            The plug-in that logs the message in its context.
	 * @param message
	 *            The message text to log.
	 * @param severity
	 *            The severity of the log message. See {@link IStatus} for valid
	 *            constants.
	 * 
	 */
	public static void log(Plugin plugin, String message, int severity) {
		log(plugin, message, severity, null);
	}

	/**
	 * Logs a message that occurred in the context of the given plug-in with the
	 * given severity and a given {@link Throwable} that originally caused this
	 * log message.
	 * 
	 * @see IStatus
	 * 
	 * @param plugin
	 *            The plug-in that logs the message in its context.
	 * @param message
	 *            The message text to log.
	 * @param severity
	 *            The severity of the log message. See {@link IStatus} for valid
	 *            constants.
	 * @param cause
	 *            The {@link Throwable} that originally caused this log message.
	 */
	public static void log(Plugin plugin, String message, int severity,
			Throwable cause) {
		String pluginId = plugin.getBundle().getSymbolicName();
		IStatus status = new Status(severity, pluginId, message, cause);
		plugin.getLog().log(status);
	}

	/**
	 * Logs an exception that occurred in the context of the given plug-in with
	 * the severity {@link IStatus#ERROR}.
	 * 
	 * @param plugin
	 *            The plug-in that logs the message in its context.
	 * @param message
	 *            The message text to log.
	 * @param cause
	 *            The {@link Throwable} that originally caused this log message.
	 */
	public static void error(Plugin plugin, String message, Throwable cause) {
		log(plugin, message, IStatus.ERROR, cause);
	}

	/**
	 * Logs an exception in a user-visible way. This includes logging with the
	 * severity {@link IStatus#ERROR} and showing a message box to the user.
	 * 
	 * @param plugin
	 *            The plug-in that logs the message in its context.
	 * @param message
	 *            The message text to log.
	 * @param cause
	 *            The {@link Throwable} that originally caused this log message.
	 */
	public static void logAndShowError(Plugin plugin, String message,
			Throwable cause) {
		error(plugin, message, cause);
		MessageUtils.showErrorAsync("An error occured", message
				+ " See the error log for details.");
	}
}